# rss.py
#
#

""" rss module. """

__copyright__ = "© 2016 Bart Thate, all rights disclaimed."

from meds.misc import slice, get_url, strip_html, need_redirect, get_feed, short_date, matching, locked
from meds.plugs.clock import Repeater
from meds.object import Object
from meds.storage import Storage
from meds.cfg import cfg
from meds.scheduler import launcher
from meds.bots import fleet
import logging
import time

seen = Object()
seen.list = []
seen.results = "rss"
seen.prefix = "results"

objs = Object()

def register(plugs): plugs.register("fetcher", fetcher)

def init(plugs):
    global seen
    global cfg
    store = Storage()
    rsscfg  = store.last("cfg", "rss")
    if rsscfg: cfg.rss = rsscfg
    rss = store.last("results", "rss")
    if rss: seen = rss
    rss = Repeater(600, fetcher, name="fetcher")
    launcher.launch(rss.start)

def display(entry):
    result = "\n"
    for key in cfg.rss.display_list:
        if key not in entry: continue
        if key not in ["link", ]: data = strip_html(entry[key])
        else: data = entry[key]
        result += "%s\n" % data
    return result.rstrip()

def fetch(obj):
    global seen
    counter = 0
    for o in get_feed(obj.rss):
        if o.link in seen.list: continue
        seen.list.append(o.link)
        s = slice(o, cfg.rss.display_list)
        s.services = "rss"
        s.prefix = "feeds"
        s.short = short_date(time.ctime())
        if not matching(cfg.rss.nosave, o.link): s.save()
        for bot in fleet: bot.announce(display(s))
        counter += 1
    return counter
    
def fetcher(event):
    result = []
    thrs = []
    store = Storage()
    for fn in store.all("rss"):
        obj = Object().load(fn)
        if "deleted" in obj and obj.deleted: continue
        if "rss" not in obj: continue
        if "http" not in obj.rss: continue
        if cfg.rss.ignore and cfg.rss.ignore in obj.rss: continue
        fetch(obj)
    seen.sync()
    return result
